﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.3
@EndOfDesignText@
' Raised when an option is selected (via user or via code).
#Event: OptionSelected(SelectedID As String)

Sub Class_Globals
	Type OptionItem(ID As String, Text As String, Checked As Boolean, Disabled As Boolean, ItemPanel As Panel, ItemCheck As ImageView, ItemLabel As Label)
	
	Private moCallback As Object
	Private panContainer As Panel
	Private msBaseEvent As String

	Private moImages(3) As Bitmap 	' 0 = unchcked, 1 = checked, 2 = disabled
	
	' Controls
	Private scvItems As ScrollView

	Private moCvs As Canvas 
	Private moSU As StringUtils
	
	Private moItems As Map
	
	' Properties
	Private miItemHeight As Int = 50dip
	Private miCheckBoxSize As Int = miItemHeight - 10dip		'5dip top, 5dip bottom
	Private miScvBackColor As Int = Colors.Transparent
	Private miScvPanColor As Int = Colors.Transparent
	Private miItemGap As Int = 3dip
	Private miLeftMargin As Int = 5dip
	Private miRightMargin As Int = 5dip
	Private miTopMargin As Int = 5dip
	Private miCheckboxMargin As Int = 5dip
	Private miBottomMargin As Int = 5dip
	Private miItemBackColor As Int = Colors.ARGB(255, 88, 88, 88)
	Private miItemForeColor As Int = Colors.White
	Private miItemDisColor As Int = Colors.Gray
	Private mfItemTextSize As Float = 22
	Private moItemFont As Typeface = Typeface.DEFAULT
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(Callback As Object, Container As Panel, BaseEvent As String, CheckedImage As Bitmap, UncheckedImage As Bitmap, DisabledImage As Bitmap)
	moCallback = Callback
	panContainer = Container
	If BaseEvent <> "" Then
		msBaseEvent = BaseEvent & "_"
	Else
		msBaseEvent = ""
	End If
	moImages(0) = CheckedImage
	moImages(1) = UncheckedImage
	moImages(2) = DisabledImage
	
	' scrollview
	scvItems.Initialize2(0, "scvItems")
	panContainer.AddView(scvItems, 0, 0, panContainer.Width, panContainer.Height)

	' our item list
	moItems.Initialize
	
	' for item label sizing
	Dim bmp As Bitmap
	bmp.InitializeMutable(1dip, 1dip)
	moCvs.Initialize2(bmp)
End Sub


'============================================================================================================
'============================================================================================================
'============================================================================================================
#Region  PUBLIC METHODS

' Adds an item to the list, does not update the display you will need to call the .Refresh method to show items that have been added.
Public Sub AddItem(ID As String, Text As String, Disabled As Boolean)
	Dim poItem As OptionItem
	If moItems.ContainsKey(ID) Then
		' just update existing
		poItem = moItems.Get(ID)
		poItem.Text = Text
		poItem.Disabled = Disabled
	Else
		poItem.Initialize
		poItem.ID = ID
		poItem.Text = Text
		poItem.Checked = False
		poItem.Disabled = Disabled
	End If
	moItems.Put(ID, poItem)
End Sub

' Removes an item from the list, does not update the display you will need to call the .Refresh method to show items that have been removed.
Public Sub RemoveItem(ID As String)
	If moItems.ContainsKey(ID) Then
		Dim poItem As OptionItem
		poItem = moItems.Get(ID)
		poItem.ItemCheck.RemoveView
		poItem.ItemLabel.RemoveView
		poItem.ItemPanel.RemoveAllViews
		poItem.ItemPanel.RemoveView
		moItems.Remove(ID)
	End If
End Sub

' Removes all items from the list, does not update the display, you will need to call the .Refresh method to show changes and resize scrollview panel.
Public Sub Clear
	Dim piIndex As Int
	
	For piIndex = moItems.Size - 1 To 0 Step -1
		Dim poItem As OptionItem
		Dim psKey As String
		psKey = moItems.GetKeyAt(piIndex)
		poItem = moItems.Get(psKey)
		If poItem.ItemCheck.IsInitialized Then
			poItem.ItemCheck.RemoveView
		End If
		If poItem.ItemLabel.IsInitialized Then
			poItem.ItemLabel.RemoveView
		End If
		If poItem.ItemPanel.IsInitialized Then
			poItem.ItemPanel.RemoveAllViews
			poItem.Itempanel.RemoveView
		End If
		moItems.Remove(psKey)
	Next
End Sub

' Re-renders all list items that have been added to the list, using the set properties.
Public Sub Refresh
	Dim piIndex As Int
	
	For piIndex = 0 To moItems.Size - 1
		Dim poItem As OptionItem
		poItem = moItems.GetValueAt(piIndex)
		If poItem.ItemCheck.IsInitialized Then
			poItem.ItemCheck.RemoveView
		End If
		If poItem.ItemLabel.IsInitialized Then
			poItem.ItemLabel.RemoveView
		End If
		If poItem.ItemPanel.IsInitialized Then
			poItem.ItemPanel.RemoveAllViews
			poItem.Itempanel.RemoveView
		End If
	Next
	scvItems.ScrollPosition = 0
	scvItems.Panel.RemoveAllViews
	scvItems.Panel.Height = miTopMargin
	
	scvItems.Color = miScvBackColor
	scvItems.Panel.Color = miScvPanColor
	
	For piIndex = 0 To moItems.Size - 1
		Dim poItem As OptionItem
		Dim psKey As String
		psKey = moItems.GetKeyAt(piIndex)
		poItem = moItems.Get(psKey)
		AddItemToScrollview(poItem)
		moItems.Put(psKey, poItem)
	Next
	scvItems.Panel.Height = scvItems.Panel.Height + miBottomMargin
	panContainer.Invalidate
End Sub


' Returns true if the item with the requested ID is checked.
Public Sub ItemIsChecked(ID As String) As Boolean
	If moItems.ContainsKey(ID) Then
		Dim poItem As OptionItem
		poItem = moItems.Get(ID)
		Return poItem.Checked
	End If
	
	Return False
End Sub

' Returns the ID of the currently checked item
Public Sub CheckedItemID() As String
	Dim piIndex As Int
	
	For piIndex = 0 To moItems.Size - 1
		Dim poItem As OptionItem = moItems.GetValueAt(piIndex)
		If poItem.Checked Then
			Return poItem.ID
		End If
	Next

	Return ""
End Sub

' Sets the requeted ID to checked, all other items to unchecked. NOTE: this method will ignore disabled items
Public Sub SetItemChecked(ID As String)
	If moItems.ContainsKey(ID) Then
		Dim poItem As OptionItem
		Dim piIndex As Int 
		poItem = moItems.Get(ID)
		If Not(poItem.Disabled) And Not(poItem.Checked) Then
			' clear all others and check this one 
			For piIndex = 0 To moItems.Size - 1
				Dim psCheckID As String = moItems.Getkeyat(piIndex)
				Dim poCheck As OptionItem = moItems.Get(psCheckID)
				If psCheckID <> ID Then
					' not our target, clear it
					poCheck.Checked = False
				Else
					' make this one checked
					poCheck.Checked = True 
				End If
				moItems.Put(psCheckID, poCheck)
				SetCheckImage(poCheck)
			Next
			RaiseChanged(poItem)
		End If
	End If
End Sub

' Returns true if the item for the requested ID is disabled
Public Sub ItemIsDisabled(ID As String) As Boolean
	If moItems.ContainsKey(ID) Then
		Dim poItem As OptionItem
		poItem = moItems.Get(ID)
		Return poItem.Disabled
	End If
	
	Return False
End Sub

' Sets the requested item for the supplied ID to disabled state (if this is the currently selected item this command is ignored!).
Public Sub SetItemDisabled(ID As String, Disabled As Boolean)
	If moItems.ContainsKey(ID) Then
		Dim poItem As OptionItem
		poItem = moItems.Get(ID)
		If Not(poItem.Checked) Then 
			poItem.Disabled = Disabled
			if Disabled then 
				poItem.Checked = false 
			End If
			SetCheckImage(poItem)
			moItems.Put(ID, poItem)
		End If 
	End If
End Sub


#End Region



'============================================================================================================
'============================================================================================================
'============================================================================================================
#Region  LOCAL EVENT HANDLERS


Private Sub panItem_Click
	Dim poSend As Panel
	Dim psKey As String

	poSend = Sender
	psKey = poSend.Tag
	SetItemChecked(psKey)
End Sub

Private Sub lblItem_Click
	Dim poSend As Label
	Dim psKey As String

	poSend = Sender
	psKey = poSend.Tag
	SetItemChecked(psKey)
End Sub

Private Sub imvItem_Click
	Dim poSend As ImageView
	Dim psKey As String

	poSend = Sender
	psKey = poSend.Tag
	SetItemChecked(psKey)
End Sub


#End Region



'============================================================================================================
'============================================================================================================
'============================================================================================================
#Region   PRIVATE METHODS 


Private Sub RaiseChanged(oItem As OptionItem)
	If moCallback <> Null Then
		If msBaseEvent <> "" Then
			If SubExists(moCallback, msBaseEvent & "OptionSelected") Then
				CallSubDelayed2(moCallback, msBaseEvent & "OptionSelected", oItem.ID)
			End If
		End If
	End If
End Sub

' You should set all item properties prior to calling this function.
Private Sub AddItemToScrollview(oItem As OptionItem)
	Dim piWidth As Int
	Dim piX As Int
	Dim piY As Int
	
	piWidth = scvItems.Width
	If scvItems.Panel.Height > miTopMargin Then
		scvItems.Panel.Height = scvItems.Panel.Height + miItemGap
	End If
	
	oItem.ItemPanel.Initialize("panItem")
	scvItems.Panel.AddView(oItem.ItemPanel, miLeftMargin, scvItems.Panel.Height, piWidth - miLeftMargin - miRightMargin, miItemHeight)
	oItem.ItemPanel.Color = miItemBackColor
	oItem.ItemPanel.Tag = oItem.ID
	
	piY = (miItemHeight - miCheckBoxSize) / 2
	oItem.ItemCheck.Initialize("imvItem")
	oItem.ItemPanel.AddView(oItem.ItemCheck, 0, piY, miCheckBoxSize, miCheckBoxSize)
	oItem.ItemCheck.Tag = oItem.ID
	oItem.ItemCheck.Gravity = Gravity.FILL
	
	oItem.ItemLabel.Initialize("lblItem")
	piX = oItem.ItemCheck.Left + oItem.ItemCheck.Width + miCheckboxMargin
	oItem.ItemPanel.AddView(oItem.ItemLabel, piX, 0, piWidth - piX - miRightMargin, miItemHeight)
	oItem.ItemLabel.Tag = oItem.ID
	oItem.ItemLabel.Text = oItem.Text
	oItem.ItemLabel.TextSize = mfItemTextSize
	oItem.ItemLabel.Typeface = moItemFont
	oItem.ItemLabel.Color = Colors.Transparent
	oItem.Itemlabel.TextColor = miItemForeColor
	oItem.ItemLabel.Gravity = Gravity.LEFT + Gravity.CENTER_VERTICAL
	oItem.ItemLabel.SingleLine = False
	SetLabelTextSize2(oItem.ItemLabel, mfItemTextSize)
	
	' set the correct checkbox image and label's text color
	SetCheckImage(oItem)

	' increase scrollview panel height
	scvItems.Panel.Height = scvItems.Panel.Height + oItem.ItemPanel.Height
End Sub

Private Sub SetCheckImage(oItem As OptionItem)
	If oItem.ItemCheck.IsInitialized Then
		If oItem.Disabled Then
			oItem.ItemCheck.SetBackgroundImage(moImages(2))
			oItem.ItemLabel.TextColor = miItemDisColor
		Else
			oItem.ItemLabel.TextColor = miItemForeColor
			If oItem.Checked Then
				oItem.ItemCheck.SetBackgroundImage(moImages(0))
			Else
				oItem.ItemCheck.SetBackgroundImage(moImages(1))
			End If
		End If
	End If
End Sub




Private Sub SetLabelTextSize2(oLbl As Label, MaxPointSize As Float)
	Dim multipleLines As Boolean = oLbl.Text.Contains(CRLF)
	Dim size As Float
	Dim r As Reflector
	Dim piPadding() As Int
	
	piPadding = oLbl.Padding
	
	r.Target = oLbl
	r.RunMethod4("setPadding", Array As Object(0,0,0,0), Array As String("java.lang.int", "java.lang.int", "java.lang.int", "java.lang.int"))
	r.RunMethod4("setIncludeFontPadding", Array As Object(False), Array As String("java.lang.boolean"))

	For size = 4 To MaxPointSize
		If CheckLblTextSize(oLbl, size, multipleLines) Then Exit
	Next
	size = size - 0.5
	' put the padding back and drop half a point to accomodate
	oLbl.Padding = piPadding
	If CheckLblTextSize(oLbl, size, multipleLines) Then size = size - 0.5
	oLbl.TextSize = size
End Sub

'returns true if the size is too large
Private Sub CheckLblTextSize(oLbl As Label, size As Float, MultipleLines As Boolean) As Boolean
	oLbl.TextSize = size
	If MultipleLines Then
		Return moSU.MeasureMultilineTextHeight(oLbl, oLbl.Text) >  oLbl.Height
	Else
		Return moCvs.MeasureStringWidth(oLbl.Text, oLbl.Typeface, size) > oLbl.Width Or _
			moSU.MeasureMultilineTextHeight(oLbl, oLbl.Text) > oLbl.Height
	End If
End Sub

#End Region




'============================================================================================================
'============================================================================================================
'============================================================================================================
#Region  PUBLIC PROPERTIES

' Gets or sets the bitmap used for the items checked state.  This method will not update the items, you will need to use the .Refresh method to observe changes.
Public Sub getBitmapChecked As Bitmap
	Return moImages(0)
End Sub
Public Sub setBitmapChecked(Value As Bitmap)
	moImages(0) = Value
End Sub

' Gets or sets the bitmap used for the items unchecked state.  This method will not update the items, you will need to use the .Refresh method to observe changes.
Public Sub getBitmapUnchecked As Bitmap
	Return moImages(1)
End Sub
Public Sub setBitmapUnchecked(Value As Bitmap)
	moImages(1) = Value
End Sub

' Gets or sets the bitmap used for the items disabled state.  This method will not update the items, you will need to use the .Refresh method to observe changes.
Public Sub getBitmapDisabled As Bitmap
	Return moImages(2)
End Sub
Public Sub setBitmapDisabled(Value As Bitmap)
	moImages(2) = Value
End Sub

' Gets or sets the height of each list item (must use .Refresh method to observe changes)
Public Sub getItemHeight As Int
	Return miItemHeight
End Sub
Public Sub setItemHeight(Value As Int)
	miItemHeight = Value
End Sub

' Gets or sets the inter-item gap in the list (must use .Refresh method to observe changes)
Public Sub getItemGap As Int
	Return miItemGap
End Sub
Public Sub setItemGap(Value As Int)
	miItemGap = Value
End Sub

' Gets or sets the background color for each list item. (Must use .Refresh method to observe change)
Public Sub getItemBackcolor As Int
	Return miItemBackColor
End Sub
Public Sub setItemBackcolor(Value As Int)
	miItemBackColor = Value
End Sub

' Gets or sets the text (foreground) color for each list item. (Must use .Refresh method to observe change)
Public Sub getItemTextcolor As Int
	Return miItemForeColor
End Sub
Public Sub setItemTextcolor(Value As Int)
	miItemForeColor = Value
End Sub

' Gets or sets the text (foreground) color for disabled list items. (Must use .Refresh method to observe change)
Public Sub getItemDisabledTextcolor As Int
	Return miItemDisColor
End Sub
Public Sub setItemDisabledTextcolor(Value As Int)
	miItemDisColor = Value
End Sub

' Gets or sets the text size for each list item. (Must use .Refresh method to observe change)
Public Sub getItemTextSize As Float
	Return mfItemTextSize
End Sub
Public Sub setItemTextSize(Value As Float)
	mfItemTextSize = Value
End Sub

' Gets or sets the typeface used to display the listitem text.  (Must use .Refresh method to observe change)
Public Sub getItemFont As Typeface
	Return moItemFont
End Sub
Public Sub setItemFont(Value As Typeface)
	moItemFont = Value
End Sub

' Gets or sets the height of each list item checkbox, should be less than .ItemHeight (must use .Refresh method to observe changes)
Public Sub getCheckboxSize As Int
	Return miCheckBoxSize
End Sub
Public Sub setCheckboxSize(Value As Int)
	miCheckBoxSize = Value
End Sub

' Gets or sets the background color for the scrollview control itself.
Public Sub getScrollviewBackcolor As Int
	Return miScvBackColor
End Sub
Public Sub setScrollviewBackcolor(Value As Int)
	miScvBackColor = Value
	scvItems.Color = Value
End Sub

' Gets or sets the scrollview panel backcolor
Public Sub getScrollviewPanelColor As Int
	Return miScvPanColor
End Sub
Public Sub setScrollviewPanelColor(Value As Int)
	miScvPanColor = Value
	scvItems.Panel.Color = Value
End Sub

' Gets or sets the left margin of items in the scrollview (must use .Refresh method to observe changes)
Public Sub getLeftMargin As Int
	Return miLeftMargin
End Sub
Public Sub setLeftMargin(Value As Int)
	miLeftMargin = Value
End Sub

' Gets or sets the right margin of items in the scrollview (must use .Refresh method to observe changes)
Public Sub getRightMargin As Int
	Return miRightMargin
End Sub
Public Sub setRightMargin(Value As Int)
	miRightMargin = Value
End Sub

' Gets or sets the top margin before the first item in the scrollview (must use .Refresh method to observe changes)
Public Sub getTopMargin As Int
	Return miTopMargin
End Sub
Public Sub setTopMargin(Value As Int)
	miTopMargin = Value
End Sub

' Gets or sets the bottom margin after the last item in the scrollview (must use .Refresh method to observe changes)
Public Sub getBottomMargin As Int
	Return miBottomMargin
End Sub
Public Sub setBottomMargin(Value As Int)
	miBottomMargin = Value
End Sub

' Gets or sets the gap after the checkbox before the item text (must use .Refresh method to observe changes)
Public Sub getCheckboxMargin As Int
	Return miCheckboxMargin
End Sub
Public Sub setCheckboxMargin(Value As Int)
	miCheckboxMargin = Value
End Sub



#End Region




